/*
 * Copyright (C) 2012 devesion.com
 * 
 * Contact: Lukas Dembinski <dembol@devesion.com>
 * Initial developer: Lukas Dembinski <dembol@devesion.com>
 * Contributor(s):
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

package com.devesion.n2io.connection.channel;

import com.devesion.n2io.connection.ConnectionException;
import java.util.ArrayList;
import java.util.List;

/**
 *
 * @author dembol
 */
class DispatcherBalancer {
    
    private static volatile DispatcherBalancer instance;
    
    /**
     * The pool of dispatchers.
     */
    private final List<Dispatcher> dispatchers;
    
    /**
     * DEFAULT_POOL_SIZE.
     */
    private static final int DEFAULT_POOL_SIZE = 10;
    
    private static int next = 0;
    
    /**
     * Constructor.
     * 
     * @throws IOException 
     */
    public DispatcherBalancer() throws ConnectionException {
        this(DEFAULT_POOL_SIZE);
    }
    
    int getPoolSize() {
        return dispatchers.size();
    }
    
    /**
     * Constructor.
     * 
     * @param poolSize the size of the pool
     * @throws IOException 
     */
    public DispatcherBalancer(int poolSize) throws ConnectionException {
        
        dispatchers = new ArrayList<Dispatcher>();
        for (int i = 0; i < poolSize; i++) {
            dispatchers.add(new Dispatcher());
        }
    }

    public static DispatcherBalancer getInstance() throws ConnectionException {
        
        if (instance != null) {
            return instance;
        }
        
        synchronized (DispatcherBalancer.class) {
            if (instance != null) {
               return instance;
            }
        
            instance = new DispatcherBalancer();
        }
        
        return instance;
    }
    
    Dispatcher getNextDispatcher() {
        final Dispatcher d = dispatchers.get(next);
        next = (next + 1) % dispatchers.size();
        return d;
    }
}
