/*
 * Copyright (C) 2012 devesion.com
 * 
 * Contact: Lukas Dembinski <dembol@devesion.com>
 * Initial developer: Lukas Dembinski <dembol@devesion.com>
 * Contributor(s):
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */


package com.devesion.n2io.connection.channel;

import com.devesion.n2io.connection.ConnectionException;
import java.io.IOException;
import java.nio.ByteBuffer;
import java.nio.channels.SocketChannel;

/**
 *
 * @author dembol
 */
public class SocketChannelConnection extends AbstractSelectableChannelConnection {

    public SocketChannelConnection(SocketChannel channel) {
        super(channel);
    }

    /**
     * Returns SocketChannel for the connection
     * 
     * @return the socketChannel
     */
    @Override
    public SocketChannel getChannel() {
        return (SocketChannel) super.getChannel();
    }

    @Override
    public int write(ByteBuffer buf) throws ConnectionException {
        try {
            final int len = getChannel().write(buf);
            if (len < 0) {
                setClosedFlag(true);
            }
            
            return len;
        } catch (IOException ex) {
            throw new ConnectionException("cannot write data to connection", ex);
        }
    }

    @Override
    public int read(ByteBuffer buf) throws ConnectionException {
        try {
            final int len = getChannel().read(buf);
            if (len < 0) {
                setClosedFlag(true);
            }
            
            return len;
        } catch (IOException ex) {
            throw new ConnectionException("cannot read data from connection", ex);
        }
    }

    @Override
    public String toString() {
        return getChannel().socket().getRemoteSocketAddress().toString();
    }
}
