/*
 * Copyright (C) 2012 devesion.com
 * 
 * Contact: Lukas Dembinski <dembol@devesion.com>
 * Initial developer: Lukas Dembinski <dembol@devesion.com>
 * Contributor(s):
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

package com.devesion.n2io.connection.decorators;

import com.devesion.n2io.connection.Connection;
import com.devesion.n2io.connection.ConnectionException;
import java.nio.ByteBuffer;
import java.nio.charset.CharacterCodingException;
import java.nio.charset.Charset;
import java.nio.charset.CharsetDecoder;

/**
 *
 * @author dembol
 */
public class TextConnection implements Connection {

    private static final int DEFAULT_BUF_CAPACITY = 1000;
    
    private Connection c;
    
    public TextConnection(Connection c) {
        this.c = c;
    }
    
    @Override
    public int read(ByteBuffer buf) throws ConnectionException {
        return c.read(buf);
    }

    @Override
    public int write(ByteBuffer buf) throws ConnectionException {
        return c.write(buf);
    
    }
    @Override
    public boolean isClosed() {
        return c.isClosed();
    }

    @Override
    public boolean isSelectable() {
        return c.isSelectable();
    }

    @Override
    public void close() throws ConnectionException {
        c.close();
    }
    
    public String readText() throws ConnectionException {
        
        final Charset charset = Charset.forName("UTF-8");
        final CharsetDecoder decoder = charset.newDecoder();
        final ByteBuffer buf = ByteBuffer.allocate(DEFAULT_BUF_CAPACITY);
        read(buf);
        
        buf.flip();
        try {
            return decoder.decode(buf).toString();
        } catch (CharacterCodingException ex) {
            throw new ConnectionException("encoding exception", ex);
        }
    }
    
    public void writeText(String text) throws ConnectionException {
        write(ByteBuffer.wrap(text.getBytes()));
    }
    
    @Override
    public String toString() {
        return c.toString();
    }
}
