/*
 * Copyright (C) 2012 devesion.com
 * 
 * Contact: Lukas Dembinski <dembol@devesion.com>
 * Initial developer: Lukas Dembinski <dembol@devesion.com>
 * Contributor(s):
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

package com.devesion.n2io.connection.selectable;

import com.devesion.n2io.connection.Connection;
import com.devesion.n2io.connection.ConnectionException;
import java.util.Set;

/**
 *
 * @author dembol
 */
public abstract class AbstractSelectableConnection implements Connection {
    
    private EventListener listener;
    
    private Object attachment;
    
    @Override
    public boolean isSelectable() {
         return true;
    }
    
    public void addEventListener(EventListener listener, Object attachment) throws ConnectionException {
        this.attachment = attachment;
        this.listener = listener;
    }
    
    public void removeEventListener() throws ConnectionException {
        listener = null;
        attachment = null;
    }
    
    public void handleEvent(Event e) {
        if (listener != null) {
            listener.handleEvent(e, attachment);
        } else {
            throw new NullPointerException("listener is null");
        }
    }
    
    public abstract void configureBlocking(boolean blocking) throws ConnectionException;

    public abstract void startGenerateEvents(Set<String> allowedEvents) throws ConnectionException;
    
    public abstract void stopGenerateEvents() throws ConnectionException;
    
    public abstract void cancelGenerateEvents() throws ConnectionException;
}
