/*
 * Copyright (C) 2012 devesion.com
 * 
 * Contact: Lukas Dembinski <dembol@devesion.com>
 * Initial developer: Lukas Dembinski <dembol@devesion.com>
 * Contributor(s):
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

package com.devesion.n2io.server;

import com.devesion.n2io.connection.Connection;
import com.devesion.n2io.connection.channel.SocketChannelConnection;
import java.io.IOException;
import java.net.InetAddress;
import java.net.InetSocketAddress;
import java.nio.channels.ServerSocketChannel;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * @author dembol
 */
public class TcpServer extends AbstractServer {
        
    private static final Logger LOGGER = LoggerFactory.getLogger(TcpServer.class);
    
    private static final int DEFAULT_SERVER_PORT = 10000;
    
    private InetSocketAddress serverAddress;
    
    private ServerSocketChannel channel = null;

    public TcpServer() {
        this(DEFAULT_SERVER_PORT);
    }

    public TcpServer(int port) {
        serverAddress = new InetSocketAddress(port);
    }
    
    public TcpServer(InetAddress address, int port) {
        serverAddress = new InetSocketAddress(address, port);
    }
    
    @Override
    protected Connection accept() throws ServerException {
        try {
            return new SocketChannelConnection(channel.accept());
        } catch (IOException e) {
            throw new ServerException(e);
        }
    }

    @Override
    protected void init() throws ServerException {
            
        LOGGER.info("Initializing TcpServer");
        
        try {
            channel = ServerSocketChannel.open();
            channel.configureBlocking(true);
            channel.socket().bind(serverAddress);
        } catch (IOException ex) {
            LOGGER.error("cannot start server socket", ex);
            throw new ServerException("cannot start server socket", ex);
        }
        
        LOGGER.info("Server started");
    }
}
