/*
 * Copyright (C) 2012 devesion.com
 * 
 * Contact: Lukas Dembinski <dembol@devesion.com>
 * Initial developer: Lukas Dembinski <dembol@devesion.com>
 * Contributor(s):
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

package com.devesion.n2io.stage;

import com.devesion.n2io.connection.Connection;
import com.devesion.n2io.connection.ConnectionException;
import com.devesion.n2io.connection.selectable.AbstractSelectableConnection;
import com.devesion.n2io.connection.selectable.Event;
import com.devesion.n2io.connection.selectable.EventListener;

/**
 *
 * @author dembol
 */
public class AsyncStage extends AbstractStage implements EventListener {
    
    private static final int DEFAULT_THREAD_NUMBER = 10;
   
    public AsyncStage(String name, int threads, AsyncStageHandler handler) {
        super(name, threads, handler);
    }

    public AsyncStage(String name, AsyncStageHandler handler) {
        this(name, DEFAULT_THREAD_NUMBER, handler);
    }

    @Override
    protected AsyncStageHandler getHandler() {
        return (AsyncStageHandler) super.getHandler();
    }
 
    @Override
    protected boolean checkConnection(StageContext ctx) throws StageException {
        
        final Connection c = ctx.getConnection();
        if (!c.isSelectable()) {
            throw new StageException("non selectable connection in async stage");
        }
        
        return super.checkConnection(ctx);
    }
    
    @Override
    protected void process(StageContext ctx) throws StageException {

        if (!checkConnection(ctx)) {
            getHandler().onEnd(ctx);
            ctx.getRouting().route(ctx, this);
            return;
        }
        
        if (!ctx.isHandled()) {
            getHandler().onStart(ctx);
            ctx.setHandled(true);
        }
        
        final AbstractSelectableConnection sc = (AbstractSelectableConnection) ctx.getConnection();
        try {
            sc.configureBlocking(false);
            sc.addEventListener(this, ctx);
            sc.startGenerateEvents(getHandler().allowedEvents());
        } catch (ConnectionException e) {
            throw new StageException("cannot add event listener", e);
        }
    }

    @Override
    public void handleEvent(Event event, Object attachment) {
        final StageContext ctx = (StageContext) attachment;
        enqueue(new AsyncStageTask(this, ctx, event));
    }

    protected void processEvent(StageContext ctx, Event event) throws StageException {

        final AbstractSelectableConnection sc = (AbstractSelectableConnection) ctx.getConnection();
        if (!checkConnection(ctx) || getHandler().onEvent(ctx, event)) {
            try {
                sc.stopGenerateEvents();
                sc.cancelGenerateEvents();
                sc.removeEventListener();
                sc.configureBlocking(true);
            } catch (ConnectionException ex) {
                throw new StageException(ex);
            }

            getHandler().onEnd(ctx);
            ctx.getRouting().route(ctx, this);
        } else {
        
            try {
                sc.startGenerateEvents(getHandler().allowedEvents());
            } catch (ConnectionException e) {
                throw new StageException("cannot add event listener", e);
            }
        }
    }
}
