/*
 * Copyright (C) 2012 devesion.com
 * 
 * Contact: Lukas Dembinski <dembol@devesion.com>
 * Initial developer: Lukas Dembinski <dembol@devesion.com>
 * Contributor(s):
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

package com.devesion.n2io.stage;

import com.devesion.n2io.connection.Connection;
import java.util.HashMap;
import java.util.Map;

/**
 *
 * @author dembol
 */
public class StageContext {

    private AbstractStage stage;
    
    private boolean handled = false;
    
    private final StageRouting routing;
      
    private final Connection connection;

    private final Map<String, Object> params = new HashMap<String, Object>();
    
    public StageContext(StageRouting routing, Connection connection) {
        this.routing = routing;
        this.connection = connection;
    }
    
    public Connection getConnection() {
        return connection;
    }

    public StageRouting getRouting() {
        return routing;
    }

    AbstractStage getStage() {
        return stage;
    }

    void setStage(AbstractStage stage) {
        this.stage = stage;
    }

    void setHandled(boolean handled) {
        this.handled = handled;
    }
    
    boolean isHandled() {
        return handled;
    }
    
    public Object getParameter(String name) {
        return params.get(name);
    }
    
    public void setParameter(String name, Object val) {
        params.put(name, val);
    }
}
