/*
 * Copyright (C) 2012 devesion.com
 * 
 * Contact: Lukas Dembinski <dembol@devesion.com>
 * Initial developer: Lukas Dembinski <dembol@devesion.com>
 * Contributor(s):
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

package com.devesion.n2io.stage;

import java.util.HashMap;
import java.util.Map;

/**
 *
 * @author dembol
 */
public class StageRouting {

    /**
     * Routing table.
     */
    private static final Map<AbstractStage, AbstractStage> ROUTE_TABLE = new HashMap<AbstractStage, AbstractStage>();

    /**
     * Add single stage route.
     * 
     * @param source source stage
     * @param destination destination stage
     */
    public final void addRoute(AbstractStage source, AbstractStage destination, String name) {
        ROUTE_TABLE.put(source, destination);
    }
    
    /**
     * Add single stage route.
     * 
     * @param source source stage
     * @param destination destination stage
     */
    public final void addRoute(AbstractStage source, AbstractStage destination) {
        ROUTE_TABLE.put(source, destination);
    }
    
    /**
     * Route a StageContext to the next stage.
     * 
     * @param source source stage
     * @param ctx processing context
     */
    final AbstractStage getRoute(AbstractStage source) {
        return ROUTE_TABLE.get(source);
    }
    
    /**
     * Route a StageContext to the next stage.
     * 
     * @param source source stage
     * @param ctx processing context
     */
    final AbstractStage getRoute(AbstractStage source, String name){
        return ROUTE_TABLE.get(source);
    }
    
    final void route(StageContext ctx, AbstractStage source) {
        AbstractStage to = getRoute(source);
        if (to != null) {
            to.enqueue(ctx);
        }
    }
        
    final void route(StageContext ctx, AbstractStage source, String name) {
        getRoute(source, name).enqueue(ctx);
    }
    
    final void selfRoute(StageContext ctx, AbstractStage source) {
        source.enqueue(ctx);
    }
}
