/*
 * Copyright (C) 2012 devesion.com
 * 
 * Contact: Lukas Dembinski <dembol@devesion.com>
 * Initial developer: Lukas Dembinski <dembol@devesion.com>
 * Contributor(s):
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

package com.devesion.n2io.samples.chat;

import com.devesion.n2io.connection.ConnectionException;
import com.devesion.n2io.connection.decorators.TextConnection;
import com.devesion.n2io.connection.selectable.Event;
import com.devesion.n2io.stage.AsyncStageHandler;
import com.devesion.n2io.stage.StageContext;
import com.devesion.n2io.stage.StageException;
import java.util.HashSet;
import java.util.Set;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 *
 * @author dembol
 */
public class ChatHandler implements AsyncStageHandler {

    private static final Logger LOGGER = LoggerFactory.getLogger(ChatHandler.class);
    
    private static Set<StageContext> contexts = new HashSet<StageContext>();

    private static Set<String> allowedEventsSet = new HashSet<String>();
    
    @Override
    public Set<String> allowedEvents() {
        return allowedEventsSet;
    }

    @Override
    public void onStart(StageContext ctx) throws StageException {
        contexts.add(ctx);
        allowedEventsSet.add(Event.EVENT_READ);
        allowedEventsSet.add(Event.EVENT_WRITE);
    }
    
    @Override
    public void onClose(StageContext ctx) throws StageException {
        contexts.remove(ctx);
    }

    @Override
    public boolean onEvent(StageContext ctx, Event e) throws StageException {
        
        if (e.getName().equals(Event.EVENT_READ)) {
            return onRead(ctx);
        } else 
        if (e.getName().equals(Event.EVENT_WRITE)) {
            LOGGER.info("test write");
            allowedEventsSet.remove(Event.EVENT_WRITE);
            // We still stay in the current stage.
            return false;
        }
        
        // Changing stage.
        return true;
    }
    
    private boolean onRead(StageContext ctx) throws StageException {
        
        final TextConnection c = new TextConnection(ctx.getConnection());
        
        try {
            final String s = c.readText();
            LOGGER.info("message '" + s.trim() + "' from " + c);
            if (s.startsWith("quit")) {
                contexts.remove(ctx);
                
                // Changing stage.
                return true;
            }
            
            for (StageContext curCtx : contexts) {

                if (curCtx.equals(ctx)) {
                    continue;
                }
                
                final TextConnection curC = new TextConnection(curCtx.getConnection());
                curC.writeText(s);
            }
            
        } catch (ConnectionException e) {
            LOGGER.error("connection error", e);
        }

        // We still stay in the current stage.
        return false;
    }
    
    @Override
    public void onEnd(StageContext ctx) throws StageException {
    }

    @Override
    public void onError(StageContext ctx, Throwable t) throws StageException {
    }
}
