/*
 * Copyright (C) 2012 devesion.com
 * 
 * Contact: Lukas Dembinski <dembol@devesion.com>
 * Initial developer: Lukas Dembinski <dembol@devesion.com>
 * Contributor(s):
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

package com.devesion.n2io.samples.chat;

import com.devesion.n2io.N2IOException;
import com.devesion.n2io.server.AbstractServer;
import com.devesion.n2io.server.TcpServer;
import com.devesion.n2io.stage.AsyncStage;
import com.devesion.n2io.stage.SyncStage;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 *
 * @author dembol
 */
public class ChatServer {

    private static final Logger LOGGER = LoggerFactory.getLogger(ChatServer.class);
   
    private ChatServer() {
        
    }
    
    /**
     * @param args
     */
    public static void main(String[] args) {

        LOGGER.info("Chat server starting");
        
        try {
            final AbstractServer server = new TcpServer(5000);
            
            final SyncStage  stage1 = new SyncStage("frontController", 10, new FrontControllerHandler());
            final AsyncStage stage2 = new AsyncStage("chat", 5, new ChatHandler());
            final SyncStage  stage3 = new SyncStage("respSend", 10, new SendResponseHandler());
                        
            server.registerStage(stage1);
            server.registerStage(stage2);
            server.registerStage(stage3);
            
            server.addRoute("frontController", "chat");
            server.addRoute("chat", "respSend");
            server.start("frontController");
        } catch (N2IOException e) {
            LOGGER.error("server error", e);
        }
    }
}
