/*
 * Copyright (C) 2012 devesion.com
 * 
 * Contact: Lukas Dembinski <dembol@devesion.com>
 * Initial developer: Lukas Dembinski <dembol@devesion.com>
 * Contributor(s):
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

package com.devesion.n2io.samples.http;

import com.devesion.n2io.N2IOException;
import com.devesion.n2io.server.AbstractServer;
import com.devesion.n2io.server.TcpServer;
import com.devesion.n2io.stage.AsyncStage;
import com.devesion.n2io.stage.SyncStage;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 *
 * @author dembol
 */
public class HttpServer {

    private static final Logger LOGGER = LoggerFactory.getLogger(HttpServer.class);

    public void start() {
        
        LOGGER.info("N2IO-http server starting");
        
        try {
            final AbstractServer server = new TcpServer(8000);
            
            final AsyncStage stage1 = new AsyncStage("read-req-stage", 5, new ReadRequestHandler());
            final SyncStage  stage2 = new SyncStage("parse-req-stage", 10, new ParseRequestHandler());
            final AsyncStage stage3 = new AsyncStage("send-resp-stage", 20, new SendResponseHandler());

            server.registerStage(stage1);
            server.registerStage(stage2);
            server.registerStage(stage3);
            
            server.addRoute("read-req-stage", "parse-req-stage");
            server.addRoute("parse-req-stage", "send-resp-stage");
            server.start("read-req-stage");
        } catch (N2IOException e) {
            LOGGER.error("server error", e);
        }
    }

    public static void main(String[] args) {
        final HttpServer s = new HttpServer();
        s.start();
    }
}
