/*
 * Copyright (C) 2012 devesion.com
 * 
 * Contact: Lukas Dembinski <dembol@devesion.com>
 * Initial developer: Lukas Dembinski <dembol@devesion.com>
 * Contributor(s):
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

package com.devesion.n2io.samples.http;

import com.devesion.n2io.stage.StageContext;
import com.devesion.n2io.stage.StageException;
import com.devesion.n2io.stage.SyncStageHandler;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 *
 * @author dembol
 */
public class ParseRequestHandler extends StandardHandler implements SyncStageHandler {

    private static final Logger LOGGER = LoggerFactory.getLogger(ParseRequestHandler.class);

    @Override
    public boolean onControl(StageContext ctx) throws StageException {
        
        final StringBuilder reqBody = (StringBuilder) ctx.getParameter("requestBody");
        
        // There should be some advanced request parsing logic
        // In this moment we only parse the first line for path to serve
        
        final int firstLineEnd = reqBody.indexOf(Constants.CRLF);
        if (firstLineEnd <= 0) {
            return true;
        }
        
        final String firstLine = reqBody.substring(0, firstLineEnd);
        final String[] arr = firstLine.split(" ");
        if (arr.length != 3) {
            return true;
        }
        
        final String path = arr[1];
        ctx.setParameter("path", path);
        LOGGER.debug("path - " + path);
        return true;
    }
}
